@@ -2,8 +2,8 @@
 import os
 
 # Replaced with the current commit when building the wheels.
-__commit__ = '23c3f67564f955af4670534e8e9d5ea19ad97e6d'
-__version__ = '0.2.2'
+__commit__ = 'ae1937f762f42bc855e3dbf82a2343856625fa23'
+__version__ = '0.2.4'
 __root_dir__ = os.path.dirname(os.path.abspath(__file__))
 
 # Keep this order to avoid cyclic imports
@@ -21,12 +21,14 @@ from sky.global_user_state import ClusterStatus
 from sky.skylet.job_lib import JobStatus
 from sky.core import (status, start, stop, down, autostop, queue, cancel,
                       tail_logs, download_logs, job_status, spot_queue,
-                      spot_status, spot_cancel, storage_ls, storage_delete)
+                      spot_status, spot_cancel, storage_ls, storage_delete,
+                      cost_report)
 
 # Aliases.
 AWS = clouds.AWS
 Azure = clouds.Azure
 GCP = clouds.GCP
+Lambda = clouds.Lambda
 Local = clouds.Local
 optimize = Optimizer.optimize
 
@@ -35,6 +37,7 @@ __all__ = [
     'AWS',
     'Azure',
     'GCP',
+    'Lambda',
     'Local',
     'Optimizer',
     'OptimizeTarget',
@@ -61,6 +64,7 @@ __all__ = [
     'stop',
     'down',
     'autostop',
+    'cost_report',
     # core APIs Job Management
     'queue',
     'cancel',
