@@ -85,9 +85,29 @@ class GCP(clouds.Cloud):
     """Google Cloud Platform."""
 
     _REPR = 'GCP'
+
+    # GCP has a 63 char limit; however, Ray autoscaler adds many
+    # characters. Through testing, this is the maximum length for the Sky
+    # cluster name on GCP.  Ref:
+    # https://cloud.google.com/compute/docs/naming-resources#resource-name-format
+    # NOTE: actually 37 is maximum for a single-node cluster which gets the
+    # suffix '-head', but 35 for a multinode cluster because workers get the
+    # suffix '-worker'. Here we do not distinguish these cases and take the
+    # lower limit.
+    _MAX_CLUSTER_NAME_LEN_LIMIT = 35
+
     _regions: List[clouds.Region] = []
     _zones: List[clouds.Zone] = []
 
+    @classmethod
+    def _cloud_unsupported_features(
+            cls) -> Dict[clouds.CloudImplementationFeatures, str]:
+        return dict()
+
+    @classmethod
+    def _max_cluster_name_length(cls) -> Optional[int]:
+        return cls._MAX_CLUSTER_NAME_LEN_LIMIT
+
     #### Regions/Zones ####
 
     @classmethod
@@ -279,9 +299,10 @@ class GCP(clouds.Cloud):
             raise
 
     @classmethod
-    def get_default_instance_type(cls) -> str:
-        # 8 vCpus, 52 GB RAM.  First-gen general purpose.
-        return 'n1-highmem-8'
+    def get_default_instance_type(cls,
+                                  cpus: Optional[str] = None) -> Optional[str]:
+        return service_catalog.get_default_instance_type(cpus=cpus,
+                                                         clouds='gcp')
 
     @classmethod
     def _get_default_region(cls) -> clouds.Region:
@@ -308,8 +329,7 @@ class GCP(clouds.Cloud):
         # --no-standard-images
         # We use the debian image, as the ubuntu image has some connectivity
         # issue when first booted.
-        image_id = service_catalog.get_image_id_from_tag(
-            'skypilot:cpu-debian-10', clouds='gcp')
+        image_id = 'skypilot:cpu-debian-10'
 
         r = resources
         # Find GPU spec, if any.
@@ -353,15 +373,13 @@ class GCP(clouds.Cloud):
                     # Though the image is called cu113, it actually has later
                     # versions of CUDA as noted below.
                     # CUDA driver version 470.57.02, CUDA Library 11.4
-                    image_id = service_catalog.get_image_id_from_tag(
-                        'skypilot:k80-debian-10', clouds='gcp')
+                    image_id = 'skypilot:k80-debian-10'
                 else:
                     # Though the image is called cu113, it actually has later
                     # versions of CUDA as noted below.
                     # CUDA driver version 510.47.03, CUDA Library 11.6
                     # Does not support torch==1.13.0 with cu117
-                    image_id = service_catalog.get_image_id_from_tag(
-                        'skypilot:gpu-debian-10', clouds='gcp')
+                    image_id = 'skypilot:gpu-debian-10'
 
         if resources.image_id is not None:
             if None in resources.image_id:
@@ -369,6 +387,9 @@ class GCP(clouds.Cloud):
             else:
                 assert region_name in resources.image_id, resources.image_id
                 image_id = resources.image_id[region_name]
+        if image_id.startswith('skypilot:'):
+            image_id = service_catalog.get_image_id_from_tag(image_id,
+                                                             clouds='gcp')
 
         assert image_id is not None, (image_id, r)
         resources_vars['image_id'] = image_id
@@ -376,45 +397,73 @@ class GCP(clouds.Cloud):
         return resources_vars
 
     def get_feasible_launchable_resources(self, resources):
-        fuzzy_candidate_list = []
         if resources.instance_type is not None:
             assert resources.is_launchable(), resources
-            return ([resources], fuzzy_candidate_list)
+            return ([resources], [])
+
+        if resources.accelerators is None:
+            # Return a default instance type with the given number of vCPUs.
+            host_vm_type = GCP.get_default_instance_type(cpus=resources.cpus)
+            if host_vm_type is None:
+                return ([], [])
+            else:
+                r = resources.copy(
+                    cloud=GCP(),
+                    instance_type=host_vm_type,
+                    accelerators=None,
+                    cpus=None,
+                )
+                return ([r], [])
+
+        use_tpu_vm = False
+        if resources.accelerator_args is not None:
+            use_tpu_vm = resources.accelerator_args.get('tpu_vm', False)
 
-        # No other resources (cpu/mem) to filter for now, so just return a
-        # default VM type.
-        host_vm_type = GCP.get_default_instance_type()
-        acc_dict = None
         # Find instance candidates to meet user's requirements
-        if resources.accelerators is not None:
-            assert len(resources.accelerators.items(
-            )) == 1, 'cannot handle more than one accelerator candidates.'
-            acc, acc_count = list(resources.accelerators.items())[0]
-            (instance_list, fuzzy_candidate_list
-            ) = service_catalog.get_instance_type_for_accelerator(
-                acc,
-                acc_count,
-                use_spot=resources.use_spot,
-                region=resources.region,
-                zone=resources.zone,
-                clouds='gcp')
+        assert len(resources.accelerators.items()
+                  ) == 1, 'cannot handle more than one accelerator candidates.'
+        acc, acc_count = list(resources.accelerators.items())[0]
 
-            if instance_list is None:
-                return ([], fuzzy_candidate_list)
-            assert len(
-                instance_list
-            ) == 1, f'More than one instance type matched, {instance_list}'
+        # For TPU VMs, the instance type is fixed to 'TPU-VM'. However, we still
+        # need to call the below function to get the fuzzy candidate list.
+        (instance_list, fuzzy_candidate_list
+        ) = service_catalog.get_instance_type_for_accelerator(
+            acc,
+            acc_count,
+            cpus=resources.cpus if not use_tpu_vm else None,
+            use_spot=resources.use_spot,
+            region=resources.region,
+            zone=resources.zone,
+            clouds='gcp')
 
+        if instance_list is None:
+            return ([], fuzzy_candidate_list)
+        assert len(
+            instance_list
+        ) == 1, f'More than one instance type matched, {instance_list}'
+
+        if use_tpu_vm:
+            host_vm_type = 'TPU-VM'
+            # FIXME(woosuk): This leverages the fact that TPU VMs have 96 vCPUs.
+            num_cpus_in_tpu_vm = 96
+            if resources.cpus is not None:
+                if resources.cpus.endswith('+'):
+                    cpus = float(resources.cpus[:-1])
+                    if cpus > num_cpus_in_tpu_vm:
+                        return ([], fuzzy_candidate_list)
+                else:
+                    cpus = float(resources.cpus)
+                    if cpus != num_cpus_in_tpu_vm:
+                        return ([], fuzzy_candidate_list)
+        else:
             host_vm_type = instance_list[0]
-            acc_dict = {acc: acc_count}
-            if resources.accelerator_args is not None:
-                use_tpu_vm = resources.accelerator_args.get('tpu_vm', False)
-                if use_tpu_vm:
-                    host_vm_type = 'TPU-VM'
+
+        acc_dict = {acc: acc_count}
         r = resources.copy(
             cloud=GCP(),
             instance_type=host_vm_type,
             accelerators=acc_dict,
+            cpus=None,
         )
         return ([r], fuzzy_candidate_list)
 
