@@ -42,7 +42,8 @@ def _ssh_control_path(ssh_control_filename: Optional[str]) -> Optional[str]:
 def ssh_options_list(ssh_private_key: Optional[str],
                      ssh_control_name: Optional[str],
                      *,
-                     timeout=30) -> List[str]:
+                     ssh_proxy_command: Optional[str] = None,
+                     timeout: int = 30) -> List[str]:
     """Returns a list of sane options for 'ssh'."""
     # Forked from Ray SSHOptions:
     # https://github.com/ray-project/ray/blob/master/python/ray/autoscaler/_private/command_runner.py
@@ -75,12 +76,20 @@ def ssh_options_list(ssh_private_key: Optional[str],
             # sky.launch().
             'ControlMaster': 'auto',
             'ControlPath': f'{_ssh_control_path(ssh_control_name)}/%C',
-            'ControlPersist': '120s',
+            'ControlPersist': '300s',
         })
     ssh_key_option = [
         '-i',
         ssh_private_key,
     ] if ssh_private_key is not None else []
+
+    if ssh_proxy_command is not None:
+        logger.debug(f'--- Proxy: {ssh_proxy_command} ---')
+        arg_dict.update({
+            # Due to how log_lib.run_with_log() works (using shell=True) we
+            # must quote this value.
+            'ProxyCommand': shlex.quote(ssh_proxy_command),
+        })
     return ssh_key_option + [
         x for y in (['-o', f'{k}={v}']
                     for k, v in arg_dict.items()
@@ -107,7 +116,8 @@ class SSHCommandRunner:
         ip: str,
         ssh_user: str,
         ssh_private_key: str,
-        ssh_control_name: str = '__default__',
+        ssh_control_name: Optional[str] = '__default__',
+        ssh_proxy_command: Optional[str] = None,
     ):
         """Initialize SSHCommandRunner.
 
@@ -124,23 +134,30 @@ class SSHCommandRunner:
                 used to avoid confliction between clusters for creating ssh
                 control files. It can simply be the cluster_name or any name
                 that can distinguish between clusters.
+            ssh_proxy_command: Optional, the value to pass to '-o
+                ProxyCommand'. Useful for communicating with clusters without
+                public IPs using a "jump server".
         """
         self.ip = ip
         self.ssh_user = ssh_user
         self.ssh_private_key = ssh_private_key
-        self.ssh_control_name = hashlib.md5(
-            ssh_control_name.encode()).hexdigest()[:_HASH_MAX_LENGTH]
+        self.ssh_control_name = (
+            None if ssh_control_name is None else hashlib.md5(
+                ssh_control_name.encode()).hexdigest()[:_HASH_MAX_LENGTH])
+        self._ssh_proxy_command = ssh_proxy_command
 
     @staticmethod
     def make_runner_list(
-            ip_list: List[str],
-            ssh_user: str,
-            ssh_private_key: str,
-            ssh_control_name: Optional[str] = None) -> List['SSHCommandRunner']:
+        ip_list: List[str],
+        ssh_user: str,
+        ssh_private_key: str,
+        ssh_control_name: Optional[str] = None,
+        ssh_proxy_command: Optional[str] = None,
+    ) -> List['SSHCommandRunner']:
         """Helper function for creating runners with the same ssh credentials"""
         return [
-            SSHCommandRunner(ip, ssh_user, ssh_private_key, ssh_control_name)
-            for ip in ip_list
+            SSHCommandRunner(ip, ssh_user, ssh_private_key, ssh_control_name,
+                             ssh_proxy_command) for ip in ip_list
         ]
 
     def _ssh_base_command(self, *, ssh_mode: SshMode,
@@ -161,7 +178,9 @@ class SSHCommandRunner:
                 ssh += ['-L', f'{remote}:localhost:{local}']
         return ssh + ssh_options_list(
             self.ssh_private_key,
-            self.ssh_control_name) + [f'{self.ssh_user}@{self.ip}']
+            self.ssh_control_name,
+            ssh_proxy_command=self._ssh_proxy_command,
+        ) + [f'{self.ssh_user}@{self.ip}']
 
     def run(
             self,
@@ -175,6 +194,7 @@ class SSHCommandRunner:
             process_stream: bool = True,
             stream_logs: bool = True,
             ssh_mode: SshMode = SshMode.NON_INTERACTIVE,
+            separate_stderr: bool = False,
             **kwargs) -> Union[int, Tuple[int, str, str]]:
         """Uses 'ssh' to run 'cmd' on a node with ip.
 
@@ -192,6 +212,7 @@ class SSHCommandRunner:
             check: Check the success of the command.
             ssh_mode: The mode to use for ssh.
                 See SSHMode for more details.
+            separate_stderr: Whether to separate stderr from stdout.
 
 
         Returns:
@@ -223,8 +244,9 @@ class SSHCommandRunner:
         command += [
             shlex.quote(f'true && source ~/.bashrc && export OMP_NUM_THREADS=1 '
                         f'PYTHONWARNINGS=ignore && ({cmd})'),
-            '2>&1',
         ]
+        if not separate_stderr:
+            command.append('2>&1')
         if not process_stream and ssh_mode == SshMode.NON_INTERACTIVE:
             command += [
                 # A hack to remove the following bash warnings (twice):
@@ -236,8 +258,8 @@ class SSHCommandRunner:
                 '; exit ${PIPESTATUS[0]}'
             ]
 
-        command = ' '.join(command)
-        command = base_ssh_command + [shlex.quote(command)]
+        command_str = ' '.join(command)
+        command = base_ssh_command + [shlex.quote(command_str)]
 
         executable = None
         if not process_stream:
@@ -294,25 +316,38 @@ class SSHCommandRunner:
         # --filter
         rsync_command.append(RSYNC_FILTER_OPTION)
 
-        # --exclude-from
-        resolved_source = pathlib.Path(source).expanduser().resolve()
-        if (resolved_source / GIT_EXCLUDE).exists():
-            # Ensure file exists; otherwise, rsync will error out.
-            rsync_command.append(
-                RSYNC_EXCLUDE_OPTION.format(str(resolved_source / GIT_EXCLUDE)))
+        if up:
+            # The source is a local path, so we need to resolve it.
+            # --exclude-from
+            resolved_source = pathlib.Path(source).expanduser().resolve()
+            if (resolved_source / GIT_EXCLUDE).exists():
+                # Ensure file exists; otherwise, rsync will error out.
+                rsync_command.append(
+                    RSYNC_EXCLUDE_OPTION.format(
+                        str(resolved_source / GIT_EXCLUDE)))
 
         ssh_options = ' '.join(
-            ssh_options_list(self.ssh_private_key, self.ssh_control_name))
+            ssh_options_list(
+                self.ssh_private_key,
+                self.ssh_control_name,
+                ssh_proxy_command=self._ssh_proxy_command,
+            ))
         rsync_command.append(f'-e "ssh {ssh_options}"')
+        # To support spaces in the path, we need to quote source and target.
+        # rsync doesn't support '~' in a quoted local path, but it is ok to
+        # have '~' in a quoted remote path.
         if up:
+            full_source_str = str(resolved_source)
+            if resolved_source.is_dir():
+                full_source_str = os.path.join(full_source_str, '')
             rsync_command.extend([
-                source,
-                f'{self.ssh_user}@{self.ip}:{target}',
+                f'{full_source_str!r}',
+                f'{self.ssh_user}@{self.ip}:{target!r}',
             ])
         else:
             rsync_command.extend([
-                f'{self.ssh_user}@{self.ip}:{source}',
-                target,
+                f'{self.ssh_user}@{self.ip}:{source!r}',
+                f'{os.path.expanduser(target)!r}',
             ])
         command = ' '.join(rsync_command)
 
