@@ -2,11 +2,12 @@
 
 import functools
 import getpass
-import inspect
 import hashlib
+import inspect
 import json
-import random
 import os
+import platform
+import random
 import re
 import socket
 import sys
@@ -15,6 +16,8 @@ from typing import Any, Dict, Optional
 import uuid
 import yaml
 
+import colorama
+
 from sky import sky_logging
 
 _USER_HASH_FILE = os.path.expanduser('~/.sky/user_hash')
@@ -324,9 +327,11 @@ def format_exception(e: Exception, use_bracket: bool = False) -> str:
     Returns:
         A string that represents the exception.
     """
+    bright = colorama.Style.BRIGHT
+    reset = colorama.Style.RESET_ALL
     if use_bracket:
-        return f'[{class_fullname(e.__class__)}]: {e}'
-    return f'{class_fullname(e.__class__)}: {e}'
+        return f'{bright}[{class_fullname(e.__class__)}]:{reset} {e}'
+    return f'{bright}{class_fullname(e.__class__)}:{reset} {e}'
 
 
 def remove_color(s: str):
@@ -352,3 +357,8 @@ def remove_file_if_exists(path: str):
     except FileNotFoundError:
         logger.debug(f'Tried to remove {path} but failed to find it. Skip.')
         pass
+
+
+def is_wsl() -> bool:
+    """Detect if running under Windows Subsystem for Linux (WSL)."""
+    return 'microsoft' in platform.uname()[3].lower()
